
{ TNamedIntegerPropEdit: Property Editor for ordinal (Integer, Longint etc.)
  properties.
  Adds an additional feature to standard TIntegerProperty Property Editor.
  It permits user to select value from a drop-down list of predefined named
  values; each line in the drop-down list has a form of Name=Value
  (e.g., Max=1000).

  Named values for this drop-down list must be supplied by creating your own
  Property Editor inherited from TNamedIntegerPropEdit and overriding its
  GetNamedValues method. For example:

    TMyNamedIntegerPropertyEditor = class(TNamedIntegerPropertyEditor)
      function GetNamedValues: string; override;
      end;

  GetNamedValues function must return string, containing a list of named values
  separated by semicolons. For example:

    function TMyNamedIntegerPropertyEditor.GetNamedValues: string;
    begin
    Result:='First=11;Second=22;Third=33'.
    end;

  Caution: when registering this Property Editor for some property of a component,
  you shoud use exact type of the property in RegisterPropertyEditor procedure
  (for example, don't attempt to use TypeInfo(Integer) for Longint property in
  Delphi 1 -- it will not work, because in 16-bit Delphi Integer is interpreted
  as Shortint, not Longint).

  If you don't know how to register Property Editors then see:
    1) demo supplied (testpe.pas, procedure Register)
    2) Delpi Help (Component Writer's Help) -- RegisterPropertyEditor
    3) excellent book written by Ray Lischner "Secrets of Delphi 2", Chapter 10.


  Author: Alexander Kopilovitch
  E-mail: aek@vib.usr.pu.ru
          aek@pcaek.spb.su

}

unit Penamint;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DsgnIntf;

type TNamedIntegerPropEdit = class(TIntegerProperty)
  private
  NamedValues: string;
  Names: TStringList;
  Values: TStringList;
  procedure CreateNamesValues;
  procedure DeleteNamesValues;
  protected
  function GetNamedValues(Component: TComponent; PropName: string): string; virtual;
  public
  procedure Activate; override;
  function GetValue: string; override;
  procedure SetValue(const Value: string); override;
  procedure GetValues(Proc: TGetStrProc); override;
  function GetAttributes: TPropertyAttributes; override;
  end;

implementation

const
      Connector = '=';
      Separator  = ';';

function TNamedIntegerPropEdit.GetNamedValues(Component: TComponent; PropName: string): string;
begin
Result:='';
end;

procedure TNamedIntegerPropEdit.CreateNamesValues;
{ creates and fill Names and Values stringlists;
  assumes following input format (in NamedValues string):
  name1=value1;name2=value2
  blanks are ignored.
}
type TReadState = (readName, readValue);
var I: Integer;
    Name, Value: string;
    C: Char;
    State: TReadState;
  procedure StartNamedValue;
  begin
  Name:='';
  Value:='';
  State:=readName;
  end;
  procedure AddToLists;
  var K: Integer;
  begin
  if Value <> '' then
    Values.Add(Value)
  else if (Names.Count = Values.Count) then begin
    { '=' is absent -- interpret string as a value (first deleting non-digits }
    if Length(Name) > 0 then for K:=1 to Length(Name) do if (Name[K] < '0') or (Name[K] > '9') then
      Delete(Name, 1, 1);
    if Name <> '' then
      Values.Add(Name);
    end
  else
    Names.Delete(Names.Count-1);
  if Values.Count > Names.Count then
    Names.Add('');
  end;
begin
Names:=TStringList.Create;
Values:=TStringList.Create;
StartNamedValue;
if Length(NamedValues) > 0 then for I:=1 to Length(NamedValues) do begin
  C:=NamedValues[I];
  case C of
    Separator:  begin
                AddToLists;
                StartNamedValue;
                end;

    Connector: begin
                Names.Add(Name);
                State:=readValue;
         end;

    else begin
      if State = readName then
        Name:=Name + C
      else if (C >= '0') and (C <= '9') then
        Value:=Value + C;
      end;
    end;
  end;
if (Name <> '') or (Value <> '') then
  AddToLists;
end;

procedure TNamedIntegerPropEdit.DeleteNamesValues;
begin
Names.Free;
Values.Free;
end;

procedure TNamedIntegerPropEdit.Activate;
begin
inherited Activate;
NamedValues:=GetNamedValues(GetComponent(0) as TComponent, GetName);
end;

function TNamedIntegerPropEdit.GetValue: string;
var I, K: Integer;
begin
CreateNamesValues;
K:=GetOrdValue;
if Values.Count > 0 then for I:=0 to Values.Count-1 do if StrToInt(Values[I]) = K then begin
  Result:=Names[I];
  if Result <> '' then
    AppendStr(Result, Connector);
  AppendStr(Result, Values[I]);
  DeleteNamesValues;
  Exit;
  end;
DeleteNamesValues;
Result:=IntToStr(K);
end;

procedure TNamedIntegerPropEdit.SetValue(const Value: string);
var I, K: Integer;
    S: string;
begin
CreateNamesValues;
K:=Pos(Connector, Value);
if K > 0 then
  S:=Copy(Value, 1, K-1)
else
  S:=Value;
S:=UpperCase(S);
if Names.Count > 0 then for I:=0 to Names.Count-1 do if S = UpperCase(Names[I]) then begin
  S:=Values[I];
  DeleteNamesValues;
  SetOrdValue(StrToInt(S));
  Exit;
  end;
DeleteNamesValues;
if K > 0 then
  S:=Copy(Value, K+1, Length(Value)-K)
else
  S:=Value;
inherited SetValue(Value);
end;

procedure TNamedIntegerPropEdit.GetValues(Proc: TGetStrProc);
var I: Integer;
    Insert_Current: Boolean;
    S: string;
begin
Insert_Current:=True;
S:=IntToStr(GetOrdValue);
CreateNamesValues;
if Values.Count > 0 then for I:=0 to Values.Count-1 do if S = Values[I] then begin
  Insert_Current:=False;
  Break;
  end;
if Insert_Current then
  Proc(S);
if Names.Count > 0 then for I:=0 to Names.Count-1 do begin
  if Names[I] <> '' then
    Proc(Names[I] + Connector + Values[I])
  else
    Proc(Values[I]);
  end;
DeleteNamesValues;
end;

function TNamedIntegerPropEdit.GetAttributes: TPropertyAttributes;
begin
Result:=inherited GetAttributes + [paValueList];
end;

end.
 